// Double squarawave music with PWM played on the STAF.
// definitions.h
// Definitions used by the code.
// 
// 28.11.2016
// Copyright (C) 2014-2016  Balthasar Szczepański
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

//RTC commands:
#define RTC_EEREAD  0x03
#define RTC_EEWRITE 0x02
#define RTC_EEWRDI  0x04
#define RTC_EEWREN  0x06
#define RTC_SRREAD  0x05
#define RTC_SRWRITE 0x01
#define RTC_READ    0x13
#define RTC_WRITE   0x12
#define RTC_UNLOCK  0x14
#define RTC_IDWRITE 0x32
#define RTC_IDREAD  0x33
#define RTC_CLRRAM  0x54

//SPI function flags:
#define ADDRFLAG  0x01
#define WRITEFLAG 0x02
#define READFLAG  0x04

//7 segment display

#if defined OLD_HARDWARE

#define SEG_A 0xdf
#define SEG_B 0xef //    aaaa
#define SEG_C 0xbf //   f    b
#define SEG_D 0x7f //   f    b
#define SEG_E 0xfd //hh  gggg
#define SEG_F 0xfb //   e    c
#define SEG_G 0xf7 //   e    c
#define SEG_H 0xfe //    dddd 

#else

#define SEG_A 0xfd
#define SEG_B 0xef //    aaaa
#define SEG_C 0xfb //   f    b
#define SEG_D 0xfe //   f    b
#define SEG_E 0xdf //hh  gggg
#define SEG_F 0x7f //   e    c
#define SEG_G 0xbf //   e    c
#define SEG_H 0xf7 //    dddd 

#endif

#define SYMB_0   (SEG_A & SEG_B & SEG_C & SEG_D & SEG_E & SEG_F        )
#define SYMB_1   (        SEG_B & SEG_C                                )
#define SYMB_2   (SEG_A & SEG_B         & SEG_D & SEG_E         & SEG_G)
#define SYMB_3   (SEG_A & SEG_B & SEG_C & SEG_D                 & SEG_G)
#define SYMB_4   (        SEG_B & SEG_C                 & SEG_F & SEG_G)
#define SYMB_5   (SEG_A         & SEG_C & SEG_D         & SEG_F & SEG_G)
#define SYMB_6   (SEG_A         & SEG_C & SEG_D & SEG_E & SEG_F & SEG_G)
#define SYMB_7   (SEG_A & SEG_B & SEG_C                                )
#define SYMB_8   (SEG_A & SEG_B & SEG_C & SEG_D & SEG_E & SEG_F & SEG_G)
#define SYMB_9   (SEG_A & SEG_B & SEG_C                 & SEG_F & SEG_G)
#define SYMB_A   (SEG_A & SEG_B & SEG_C         & SEG_E & SEG_F & SEG_G)
#define SYMB_B   (                SEG_C & SEG_D & SEG_E & SEG_F & SEG_G)
#define SYMB_C   (                        SEG_D & SEG_E         & SEG_G)
#define SYMB_D   (        SEG_B & SEG_C & SEG_D & SEG_E         & SEG_G)
#define SYMB_E   (SEG_A                 & SEG_D & SEG_E & SEG_F & SEG_G)
#define SYMB_F   (SEG_A                         & SEG_E & SEG_F & SEG_G)
#define SYMB_NUL (0xFF)
#define SYMB_o   (SEG_A & SEG_B                         & SEG_F & SEG_G)
#define SYMB_n   (SEG_A & SEG_B                         & SEG_F        )
#define SYMB_g   (SEG_A & SEG_B & SEG_C & SEG_D         & SEG_F & SEG_G)
#define SYMB_U   (                SEG_C & SEG_D & SEG_E                )
#define SYMB_L   (                        SEG_D & SEG_E & SEG_F        )
#define SYMB_M1  (SEG_A & SEG_B                 & SEG_E & SEG_F        )
#define SYMB_M2  (SEG_A & SEG_B & SEG_C                 & SEG_F        )
#define SYMB_I   (                                SEG_E                )  
#define SYMB_MIN (                                                SEG_G)
#define SYMB_DOT (SEG_H)

//pitch coding

#define CC0 1433
#define CD0 1353
#define DD0 1277
#define DE0 1205
#define EE0 1138
#define FF0 1074
#define FG0 1014
#define GG0  957
#define GA0  903
#define AA0  852//27.5
#define AH0  804
#define HH0  759

#define CC1  717
#define CD1  676
#define DD1  638
#define DE1  603
#define EE1  569
#define FF1  537
#define FG1  507
#define GG1  478
#define GA1  451
#define AA1  426//55
#define AH1  402
#define HH1  380
              
#define CC2  358
#define CD2  338
#define DD2  319
#define DE2  301
#define EE2  284
#define FF2  268
#define FG2  253
#define GG2  239
#define GA2  226
#define AA2  213//110
#define AH2  201
#define HH2  190
              
#define CC3  179
#define CD3  169
#define DD3  160
#define DE3  151
#define EE3  142
#define FF3  134
#define FG3  127
#define GG3  120
#define GA3  113
#define AA3  107//220
#define AH3  101
#define HH3   95
              
#define CC4   90
#define CD4   85
#define DD4   80
#define DE4   75
#define EE4   71
#define FF4   67
#define FG4   63
#define GG4   60
#define GA4   56
#define AA4   53//440
#define AH4   50
#define HH4   47

#define CC5   45
#define CD5   42
#define DD5   40
#define DE5   38
#define EE5   36
#define FF5   34
#define FG5   32
#define GG5   30
#define GA5   28
#define AA5   27//880
#define AH5   25
#define HH5   24

// // NOT recommended:

// #define CC6   22
// #define CD6   21
// #define DD6   20
// #define DE6   19
// #define EE6   18
// #define FF6   17
// #define FG6   16
// #define GG6   15
// #define GA6   14
// #define AA6   13//1760
// #define AH6   12
// #define HH6   11

// // completely worthless:

// #define CC7   11
// #define CD7   11
// #define DD7   10
// #define DE7    9
// #define EE7    9
// #define FF7    8
// #define FG7    8
// #define GG7    7
// #define GA7    7
// #define AA7    7//3520
// #define AH7    6
// #define HH7    6


#define SIL    0//silence

#define PTCH 0x07ff

//length coding

#define N16  0x0000
#define D16  0x1000
#define N8   0x2000
#define D8   0x3000
#define N4   0x4000
#define D4   0x5000
#define N2   0x6000
#define D2   0x7000
#define N1   0x8000
#define D1   0x9000

#define NXSP 0xA000//next tempo
#define REPL 0xB000//start from beginning
#define STOP 0xC000//stop
#define RPST 0xD000//repeat start
#define RPND 0xE000//repeat stop
#define RSV3 0xF000//reserved

#define LENG 0xF000

//continuity coding

#define SNGL 0x0800
#define CONT 0x0000

//repeats

#define RP0 0x0000
#define RP1 0x0100
#define RP2 0x0200
#define RP3 0x0300

#define STOP_OFFSET 164 //~0.04s

#define ALARMVALUE 0xa5

unsigned char bcd_add_byte (unsigned char, unsigned char);
void          lpm3         ();
